<?php

namespace Drp\Peek\Environment;

use Dotenv\Dotenv;
use Dotenv\Environment\Adapter\EnvConstAdapter;
use Dotenv\Environment\Adapter\PutenvAdapter;
use Dotenv\Environment\Adapter\ServerConstAdapter;
use Dotenv\Environment\DotenvFactory;
use Drp\Peek\PeekFacade;
use Illuminate\Contracts\Foundation\Application;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\File;

class Environment
{
    public function __construct(Application $app)
    {
        $this->app = $app;
        $this->bannedVariables = new BannedVariables($app['config']->get('peek.env_variables.banned'));
        $this->dotEnv = $this->loadDotEnv($app);
    }

    public function variables(): Collection
    {
        return collect($this->getEnvironmentVariableNames())
            ->map(function ($envVariable) {
                if (isset($_ENV[$envVariable]) === false) {
                    return null;
                }

                $variable = new Variable($envVariable, $_ENV[$envVariable]);
                if ($this->bannedVariables->isBanned($envVariable)) {
                    return $variable->banned();
                }

                return $variable;
            })
            ->filter()
            ->sortBy(function (Variable $variable): string {
                return $variable->name();
            });
    }

    public function write(array $variables): void
    {
        (new Writer)->put($variables, $this->bannedVariables);

        if (File::exists($this->app->getCachedConfigPath()) === false) {
            return;
        }

        $this->loadDotEnv($this->app);
        $resource = fopen('php://memory', 'w+');
        PeekFacade::writeCommandToStream('config:cache', [], $resource);
        fclose($resource);
    }

    /**
     * @return \Dotenv|\Dotenv\Dotenv
     */
    protected function loadDotEnv(Application $app)
    {
        if ($app->version() < '5.6') {
            return $this->makeDotEnvMutable(function () use ($app) {
                return \Dotenv::load($app->environmentPath(), $app->environmentFile());
            });
        }

        return tap($this->makeDotEnv($app), function (Dotenv $dotEnv): void {
            $dotEnv->overload();
        });
    }

    private function getEnvironmentVariableNames(): array
    {
        if ($this->app->version() < '5.8') {
            $matches = [];
            preg_match_all(
                '/([A-z0-9]+)\=/m',
                File::get($this->app->environmentPath() . '/' . $this->app->environmentFile()),
                $matches
            );

            return $matches[1];
        }

        return $this->dotEnv->getEnvironmentVariableNames();
    }

    protected function makeDotEnv(Application $app): Dotenv
    {
        if ($app->version() >= '5.6' && $app->version() < '5.8') {
            return new Dotenv($app->environmentPath(), $app->environmentFile());
        }

        return Dotenv::create(
            $app->environmentPath(),
            $app->environmentFile(),
            new DotenvFactory([new EnvConstAdapter, new ServerConstAdapter, new PutenvAdapter])
        );
    }

    /**
     * @return \Dotenv\Dotenv|null|mixed
     */
    private function makeDotEnvMutable(callable $callback)
    {
        \Dotenv::makeMutable();
        $result = $callback();
        \Dotenv::makeImmutable();

        return $result;
    }
}
